# Generated by h2py from /usr/include/sys/cdio.h
CDROM_LBA = 0x01
CDROM_MSF = 0x02
CDROM_DATA_TRACK = 0x04
CDROM_LEADOUT = 0xAA
CDROM_AUDIO_INVALID = 0x00
CDROM_AUDIO_PLAY = 0x11
CDROM_AUDIO_PAUSED = 0x12
CDROM_AUDIO_COMPLETED = 0x13
CDROM_AUDIO_ERROR = 0x14
CDROM_AUDIO_NO_STATUS = 0x15
CDROM_DA_NO_SUBCODE = 0x00
CDROM_DA_SUBQ = 0x01
CDROM_DA_ALL_SUBCODE = 0x02
CDROM_DA_SUBCODE_ONLY = 0x03
CDROM_XA_DATA = 0x00
CDROM_XA_SECTOR_DATA = 0x01
CDROM_XA_DATA_W_ERROR = 0x02
CDROM_BLK_512 = 512
CDROM_BLK_1024 = 1024
CDROM_BLK_2048 = 2048
CDROM_BLK_2056 = 2056
CDROM_BLK_2336 = 2336
CDROM_BLK_2340 = 2340
CDROM_BLK_2352 = 2352
CDROM_BLK_2368 = 2368
CDROM_BLK_2448 = 2448
CDROM_BLK_2646 = 2646
CDROM_BLK_2647 = 2647
CDROM_BLK_SUBCODE = 96
CDROM_NORMAL_SPEED = 0x00
CDROM_DOUBLE_SPEED = 0x01
CDROM_QUAD_SPEED = 0x03
CDROM_TWELVE_SPEED = 0x0C
CDROM_MAXIMUM_SPEED = 0xff
CDIOC = (0x04 << 8)
CDROMPAUSE = (CDIOC|151)
CDROMRESUME = (CDIOC|152)
CDROMPLAYMSF = (CDIOC|153)
CDROMPLAYTRKIND = (CDIOC|154)
CDROMREADTOCHDR = (CDIOC|155)
CDROMREADTOCENTRY = (CDIOC|156)
CDROMSTOP = (CDIOC|157)
CDROMSTART = (CDIOC|158)
CDROMEJECT = (CDIOC|159)
CDROMVOLCTRL = (CDIOC|160)
CDROMSUBCHNL = (CDIOC|161)
CDROMREADMODE2 = (CDIOC|162)
CDROMREADMODE1 = (CDIOC|163)
CDROMREADOFFSET = (CDIOC|164)
CDROMGBLKMODE = (CDIOC|165)
CDROMSBLKMODE = (CDIOC|166)
CDROMCDDA = (CDIOC|167)
CDROMCDXA = (CDIOC|168)
CDROMSUBCODE = (CDIOC|169)
CDROMGDRVSPEED = (CDIOC|170)
CDROMSDRVSPEED = (CDIOC|171)
SCMD_READ_TOC = 0x43
SCMD_PLAYAUDIO_MSF = 0x47
SCMD_PLAYAUDIO_TI = 0x48
SCMD_PAUSE_RESUME = 0x4B
SCMD_READ_SUBCHANNEL = 0x42
SCMD_PLAYAUDIO10 = 0x45
SCMD_PLAYTRACK_REL10 = 0x49
SCMD_READ_HEADER = 0x44
SCMD_PLAYAUDIO12 = 0xA5
SCMD_PLAYTRACK_REL12 = 0xA9
SCMD_CD_PLAYBACK_CONTROL = 0xC9
SCMD_CD_PLAYBACK_STATUS = 0xC4
SCMD_READ_CDDA = 0xD8
SCMD_READ_CDXA = 0xDB
SCMD_READ_ALL_SUBCODES = 0xDF
CDROM_MODE2_SIZE = 2336
